from collections import namedtuple

from durations_nlp.parser import extract_tokens
from durations_nlp.scales import Scale
from durations_nlp.exceptions import ScaleFormatError
from durations_nlp.constants import *


DurationRepresentation = namedtuple(
    'DurationRepresentation',
    ['value', 'scale']
)


class Duration(object):
    """Duration representation class

    Duration objects parses an input duration representation
    string, and provides a set of methods to retrieve and
    convert it's value to other units.

    Example:
        >>> d = Duration('1m')
        >>> d.to_seconds()
        60.0
        >>> d.to_hours()
        0.02
        >>> d = Duration('2d 3 hours')
        >>> d.to_minutes()
        3060.0
    """

    def __init__(self, representation, *args, **kwargs):
        self.representation = representation
        self.parsed_durations = self.parse(self.representation)
        self.seconds = self._compute_seconds_value()

    def __str__(self):
        return '<Duration {0}>'.format(self.representation)

    def __repr__(self):
        return self.__str__()

    def _compute_seconds_value(self):
        seconds = 0

        for duration in self.parsed_durations:
            seconds += duration.value * duration.scale.conversion_unit

        return seconds

    def parse(self, representation):
        """Parses a duration string representation


        :param  representation: duration as a string, example: '1d' (day),
                                '34minutes' (minutes), '485s' (seconds)...
        :type   representation: string

        :returns: the parsed duration representation
        :rtype: DurationRepresentation
        """
        elements = extract_tokens(representation)

        try:
            scales = [DurationRepresentation(float(p[0]), Scale(p[1]))
                      for p in elements]
        except ValueError:
            raise ScaleFormatError("Malformed duration representation: {0}".format(representation))  # noqa: E501

        return scales

    def to_centuries(self):
        return round(self.seconds / float(SCALE_CENTURY_CONVERSION_UNIT), 2)

    def to_decades(self):
        return round(self.seconds / float(SCALE_DECADE_CONVERSION_UNIT), 2)

    def to_years(self):
        return round(self.seconds / float(SCALE_YEAR_CONVERSION_UNIT), 2)

    def to_months(self):
        return round(self.seconds / float(SCALE_MONTH_CONVERSION_UNIT), 2)

    def to_weeks(self):
        return round(self.seconds / float(SCALE_WEEK_CONVERSION_UNIT), 2)

    def to_days(self):
        return round(self.seconds / float(SCALE_DAY_CONVERSION_UNIT), 2)

    def to_hours(self):
        return round(self.seconds / float(SCALE_HOUR_CONVERSION_UNIT), 2)

    def to_minutes(self):
        return round(self.seconds / float(SCALE_MINUTE_CONVERSION_UNIT), 2)

    def to_seconds(self):
        return round(self.seconds / float(SCALE_SECOND_CONVERSION_UNIT), 2)

    def to_miliseconds(self):
        return round(self.seconds / float(SCALE_MILISECOND_CONVERSION_UNIT), 2)



from .scales import Scale  # noqa: F401
from .duration import Duration  # noqa: F401
version = (0, 3, 3)

__title__ = "durations"
__author__ = "Oleiade"
__license__ = "MIT"

__version__ = '.'.join(map(str, version))



from durations_nlp.scales import Scale
from durations_nlp.constants import *
from durations_nlp.exceptions import ScaleFormatError, InvalidTokenError


def valid_token(token):
    """Asserts a provided string is a valid duration token representation

    :param  token: duration representation token
    :type   token: string
    """
    is_scale = False

    # Check if the token represents a scale
    # If it doesn't set a flag accordingly
    try:
        Scale(token)
        is_scale = True
    except ScaleFormatError:
        pass

    # If token either represents a numerical value, a
    # separator token, or a scale, it is considered valid
    if any([token.isdigit(), token in SEPARATOR_TOKENS, is_scale]):
        return True

    return False


def compute_char_token(c):
    if c.isdigit():
        return SCALE_TOKEN_DIGIT
    elif c.isalpha():
        return SCALE_TOKEN_ALPHA

    return None


def extract_tokens(representation, separators=SEPARATOR_CHARACTERS):
    """Extracts durations tokens from a duration representation.

    Parses the string representation incrementaly and raises
    on first error met.

    :param  representation: duration representation
    :type   representation: string
    """
    buff = ""
    elements = []
    last_token = None

    for index, c in enumerate(representation):
        # if separator character is found, push
        # the content of the buffer in the elements list
        if c in separators:
            if buff:
                # If the last found token is invalid,
                # raise and InvalidTokenError
                if not valid_token(buff):
                    raise InvalidTokenError(
                        "Duration representation {0} contains "
                        "an invalid token: {1}".format(representation, buff)
                    )

                # If buffer content is a separator word, for example
                # "and", just ignore it
                if not buff.strip() in SEPARATOR_TOKENS:
                    elements.append(buff)

            # Anyway, reset buffer and last token marker
            # to their zero value
            buff = ""
            last_token = None
        else:
            token = compute_char_token(c)
            if token is not None \
                    and last_token is not None \
                    and token != last_token:
                elements.append(buff)
                buff = c
            else:
                buff += c

            last_token = token

    # push the content left in representation
    # in the elements list
    elements.append(buff)

    return list(zip(elements[::2], elements[1::2]))



from durations_nlp import Duration
from durations_nlp.exceptions import ScaleFormatError


def valid_duration(representation):
    try:
        Duration(representation)
    except ScaleFormatError:
        return False

    return True



from durations_nlp.constants import *
from durations_nlp.exceptions import ScaleFormatError


class Scale(object):
    """Duration scale representation class

    :param  representation: scale string representation
    :type   representation: string
    """
    # Existing scales representation listing
    SCALES = (
        SCALE_CENTURY,
        SCALE_DECADE,
        SCALE_YEAR,
        SCALE_MONTH,
        SCALE_WEEK,
        SCALE_DAY,
        SCALE_HOUR,
        SCALE_MINUTE,
        SCALE_SECOND,
        SCALE_MILISECOND
    )

    # Scale short string representation to seconds amount per unit
    # conversion table
    SCALES_CONVERTION_UNITS = {
        SCALE_CENTURY.short: SCALE_CENTURY_CONVERSION_UNIT,
        SCALE_DECADE.short: SCALE_DECADE_CONVERSION_UNIT,
        SCALE_YEAR.short: SCALE_YEAR_CONVERSION_UNIT,
        SCALE_MONTH.short: SCALE_MONTH_CONVERSION_UNIT,
        SCALE_WEEK.short: SCALE_WEEK_CONVERSION_UNIT,
        SCALE_DAY.short: SCALE_DAY_CONVERSION_UNIT,
        SCALE_HOUR.short: SCALE_HOUR_CONVERSION_UNIT,
        SCALE_MINUTE.short: SCALE_MINUTE_CONVERSION_UNIT,
        SCALE_SECOND.short: SCALE_SECOND_CONVERSION_UNIT,
        SCALE_MILISECOND.short: SCALE_MILISECOND_CONVERSION_UNIT
    }

    def __init__(self, str_representation, *args, **kwargs):
        self.representation = self.get(str_representation)
        self.conversion_unit = self.SCALES_CONVERTION_UNITS[self.representation.short]  # noqa: E501

    def __str__(self):
        return '<Scale {0}>'.format(self.representation.long_singular)

    def __repr__(self):
        return self.__str__()

    def get(self, str_representation):
        """Retrieves a scale representation from it's string representation

        :param  str_representation: scale string representation to be retrieved
        :type   str_representation: string

        :raises: ScaleFormatError
        :returns: scale representation
        :rtype: ScaleRepresentation
        """
        for scale in self.SCALES:
            if str_representation in scale:
                return scale
        raise ScaleFormatError("Unsupported scale format: {0}".format(str_representation))  # noqa: E501



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['durations_nlp']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'durations-nlp',
    'version': '0.4.0',
    'description': 'A python durations parsing library.',
    'long_description': '# durations_nlp\n\n[![CircleCI](https://circleci.com/gh/timwedde/durations_nlp.svg?style=svg)](https://circleci.com/gh/timwedde/durations_nlp)\n[![codecov](https://codecov.io/gh/timwedde/durations_nlp/branch/master/graph/badge.svg)](https://codecov.io/gh/timwedde/durations_nlp)\n[![Downloads](https://pepy.tech/badge/durations_nlp)](https://pepy.tech/project/durations_nlp)\n\nA python durations parsing library, providing a straight-forward API to parse duration string representations such as `1d`, `1 day 2 hours` or `2 days 3h 26m 52s` and convert them to numeric values.\n\n## What and Why\nIt\'s easier and more straight forward to read a duration expressed in natural language (at least for a human), as an expression rather than an amount. When writing configuration files for example:\n\n```yaml\ninterval: 3 hours\n```\n\nis easier to understand for a human than\n\n```yaml\ninterval: 10800  # seconds\n```\n\n## Installation\n\n`durations_nlp` can be installed via pip:\n```bash\n$ pip install durations_nlp\n```\n\n## Usage\nTo parse a duration string representation, just instantiate a Duration object and let it work for you. A Duration representation is composed of as many ``<value><scale>`` pairs as you need to express it:\n* A value is an integer amount.\n* A scale is a duration unit in it\'s short or long form (both singular and plural).\n* Duration pairs can be separated with sep characters and expressions such as `,` or `and`\n\n### Example Input\n\n* `1d`\n* `2 days`\n* `2 days and 4 hours`\n* `4M, 22d and 6hours`\n\n### Scales Reference\n\n* Century: `c`, `century`, `centuries`\n* Decade: `D`, `decade`, `decades`\n* Year: `y`, `year`, `Year`\n* Month: `M`, `month`, `months`\n* Week: `w`, `week`, `weeks`\n* Day: `d`, `day`, `days`\n* Hour: `h`, `hour`, `hours`\n* Minute:`m`, `minute`, `minutes`\n* Second: `s`, `second`, `seconds`\n* Milisecond: `ms`, `milisecond`, `miliseconds`\n\n### Usage Example\n\n```python\nfrom durations_nlp import Duration\n\none_hour = "1hour"\n\none_hour_duration = Duration(one_hour)\none_hour_duration.to_seconds()\n# >>> 3600.0\none_hour_duration.to_minutes()\n# >>> 60.0\n\n# You can even compose durations in their short\n# and long variations\ntwo_days_three_hours = "2 days, 3h"\ntwo_days_three_hours_duration = Duration(two_days_three_hours)\ntwo_days_three_hours_duration.to_seconds()\n# >>> 183600.0\ntwo_days_three_hours_duration.to_hours()\n# >>> 51.0\n```\n',
    'author': 'Tim Wedde',
    'author_email': 'timwedde@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/timwedde/durations_nlp',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



from collections import namedtuple

# Scales separator chars and tokens
SEPARATOR_CHARACTERS = (
    " ",
    "\t",
    ","
)

SEPARATOR_TOKENS = (
    'and',
)

# Scale parsing tokens definition
SCALE_TOKEN_ALPHA = 0
SCALE_TOKEN_DIGIT = 1


ScaleRepresentation = namedtuple(
    'ScaleRepresentation',
    ['short', 'long_singular', 'long_plural']
)

# Duration scales string representations
SCALE_CENTURY = ScaleRepresentation('c', 'century', 'centuries')
SCALE_DECADE = ScaleRepresentation('D', 'decade', 'decades')
SCALE_YEAR = ScaleRepresentation('y', 'year', 'Year')
SCALE_MONTH = ScaleRepresentation('M', 'month', 'months')
SCALE_WEEK = ScaleRepresentation('w', 'week', 'weeks')
SCALE_DAY = ScaleRepresentation('d', 'day', 'days')
SCALE_HOUR = ScaleRepresentation('h', 'hour', 'hours')
SCALE_MINUTE = ScaleRepresentation('m', 'minute', 'minutes')
SCALE_SECOND = ScaleRepresentation('s', 'second', 'seconds')
SCALE_MILISECOND = ScaleRepresentation('ms', 'milisecond', 'miliseconds')

# Conversion constants. Each contains it's seconds
# per unit amount. For example: 1 minute is 60 seconds,
# or 1 hour is 3600 seconds.
SCALE_SECOND_CONVERSION_UNIT = 1
SCALE_MILISECOND_CONVERSION_UNIT = 0.001 * SCALE_SECOND_CONVERSION_UNIT
SCALE_MINUTE_CONVERSION_UNIT = 60 * SCALE_SECOND_CONVERSION_UNIT
SCALE_HOUR_CONVERSION_UNIT = 60 * SCALE_MINUTE_CONVERSION_UNIT
SCALE_DAY_CONVERSION_UNIT = 24 * SCALE_HOUR_CONVERSION_UNIT
SCALE_WEEK_CONVERSION_UNIT = 7 * SCALE_DAY_CONVERSION_UNIT
SCALE_MONTH_CONVERSION_UNIT = 30.5 * SCALE_DAY_CONVERSION_UNIT
SCALE_YEAR_CONVERSION_UNIT = 12 * SCALE_MONTH_CONVERSION_UNIT
SCALE_DECADE_CONVERSION_UNIT = 10 * SCALE_YEAR_CONVERSION_UNIT
SCALE_CENTURY_CONVERSION_UNIT = 10 * SCALE_DECADE_CONVERSION_UNIT
